@@ -44,6 +44,8 @@ enum {
     TI_COLUMN_MEMBERS_LIST_HANDLE = 0,
     TI_COLUMN_MEMBERS_LIST_NAME,
     TI_COLUMN_MEMBERS_LIST_STATUS,
+    TI_COLUMN_MEMBERS_LIST_OWNER,
+    TI_COLUMN_MEMBERS_LIST_OWNER_NAME,
     TI_COLUMN_MEMBERS_LIST_N_COLUMNS
 };
 
@@ -144,148 +146,167 @@ ti_page_channel_group_dispose (GObject *object)
       priv->preferences = NULL;
   }
 
-TI_OBJ_UNREF (priv->handle_mapper);
-TI_OBJ_UNREF (priv->dialog_add_members);
-TI_OBJ_UNREF (priv->dialog_get_text);
-TI_OBJ_UNREF (priv->members_list);
+  TI_OBJ_UNREF (priv->handle_mapper);
+  TI_OBJ_UNREF (priv->dialog_add_members);
+  TI_OBJ_UNREF (priv->dialog_get_text);
+  TI_OBJ_UNREF (priv->members_list);
 
-G_OBJECT_CLASS (ti_page_channel_group_parent_class)->dispose (object);
+  G_OBJECT_CLASS (ti_page_channel_group_parent_class)->dispose (object);
 }
 
 static void
 ti_page_channel_group_class_init (TIPageChannelGroupClass *klass)
 {
-GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-TIPageClass *page_class = TI_PAGE_CLASS (klass);
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  TIPageClass *page_class = TI_PAGE_CLASS (klass);
 
-gobject_class->dispose = ti_page_channel_group_dispose;
+  gobject_class->dispose = ti_page_channel_group_dispose;
 
-page_class->setup_page = _ti_page_channel_group_setup_page;
+  page_class->setup_page = _ti_page_channel_group_setup_page;
 
-g_type_class_add_private (klass, sizeof (TIPageChannelGroupPrivate));
+  g_type_class_add_private (klass, sizeof (TIPageChannelGroupPrivate));
 }
 
 static void
 ti_page_channel_group_init (TIPageChannelGroup *self)
 {
-TIPageChannelGroupPrivate *priv = TI_PAGE_CHANNEL_GROUP_GET_PRIVATE (self);
+  TIPageChannelGroupPrivate *priv = TI_PAGE_CHANNEL_GROUP_GET_PRIVATE (self);
 
-priv->channel = NULL;
-priv->preferences = ti_preferences_new ();
-priv->handle_mapper = NULL;
-priv->dialog_add_members = NULL;
-priv->dialog_get_text = NULL;
-priv->members_list = NULL;
+  priv->channel = NULL;
+  priv->preferences = ti_preferences_new ();
+  priv->handle_mapper = NULL;
+  priv->dialog_add_members = NULL;
+  priv->dialog_get_text = NULL;
+  priv->members_list = NULL;
 }
 
 static void
 _ti_page_channel_group_members_changed (TpChannel *channel,
-                                      const gchar *message,
-                                      const GArray *added,
-                                      const GArray *removed,
-                                      const GArray *local_pending,
-                                      const GArray *remote_pending,
-                                      guint actor,
-                                      guint reason,
-                                      gpointer unused,
-                                      GObject *weak_object)
+                                        const gchar *message,
+                                        const GArray *added,
+                                        const GArray *removed,
+                                        const GArray *local_pending,
+                                        const GArray *remote_pending,
+                                        guint actor,
+                                        guint reason,
+                                        gpointer unused,
+                                        GObject *weak_object)
 {
-TIPageChannelGroup *self = TI_PAGE_CHANNEL_GROUP (weak_object);
-g_assert (added != NULL);
-g_assert (local_pending != NULL);
-g_assert (remote_pending != NULL);
+  TIPageChannelGroup *self = TI_PAGE_CHANNEL_GROUP (weak_object);
 
-_ti_page_channel_group_log_members_changed (self, message, added, removed,
-    local_pending, remote_pending, actor, reason);
+  g_assert (added != NULL);
+  g_assert (local_pending != NULL);
+  g_assert (remote_pending != NULL);
 
-_ti_page_channel_group_append_members_to_list (self, added,
-    TI_GROUP_MEMBER_STATUS_REGULAR);
-_ti_page_channel_group_append_members_to_list (self, local_pending,
-    TI_GROUP_MEMBER_STATUS_LOCAL_PENDING);
-_ti_page_channel_group_append_members_to_list (self, remote_pending,
-    TI_GROUP_MEMBER_STATUS_REMOTE_PENDING);
+  _ti_page_channel_group_log_members_changed (self, message, added, removed,
+      local_pending, remote_pending, actor, reason);
 
-_ti_page_channel_group_remove_members (self, removed);
+  _ti_page_channel_group_append_members_to_list (self, added,
+      TI_GROUP_MEMBER_STATUS_REGULAR);
+  _ti_page_channel_group_append_members_to_list (self, local_pending,
+      TI_GROUP_MEMBER_STATUS_LOCAL_PENDING);
+  _ti_page_channel_group_append_members_to_list (self, remote_pending,
+      TI_GROUP_MEMBER_STATUS_REMOTE_PENDING);
+
+  _ti_page_channel_group_remove_members (self, removed);
 }
 
 static void
 _ti_page_channel_group_group_flags_changed (TpChannel *channel,
-                                          guint added,
-                                          guint removed,
-                                          gpointer unused,
-                                          GObject *weak_object)
+                                            guint added,
+                                            guint removed,
+                                            gpointer unused,
+                                            GObject *weak_object)
 {
-TIPageChannelGroup *self = TI_PAGE_CHANNEL_GROUP (weak_object);
-TIPageChannelGroupPrivate *priv = TI_PAGE_CHANNEL_GROUP_GET_PRIVATE (self);
-gboolean can_add;
+  TIPageChannelGroup *self = TI_PAGE_CHANNEL_GROUP (weak_object);
+  TIPageChannelGroupPrivate *priv = TI_PAGE_CHANNEL_GROUP_GET_PRIVATE (self);
+  gboolean can_add;
 
-_ti_page_channel_group_log_group_flags_changed (self, added,
-    removed);
+  _ti_page_channel_group_log_group_flags_changed (self, added,
+      removed);
 
-priv->group_flags = (priv->group_flags | added) & (~removed);
+  priv->group_flags = (priv->group_flags | added) & (~removed);
 
-_ti_page_channel_group_refresh_group_flags_checkboxes (self);
+  _ti_page_channel_group_refresh_group_flags_checkboxes (self);
 
-can_add = (priv->group_flags & TP_CHANNEL_GROUP_FLAG_CAN_ADD) != 0;
-gtk_widget_set_sensitive (GTK_WIDGET (priv->button_remove_members), can_add);
+  can_add = (priv->group_flags & TP_CHANNEL_GROUP_FLAG_CAN_ADD) != 0;
+  gtk_widget_set_sensitive (GTK_WIDGET (priv->button_remove_members), can_add);
 
-_ti_page_channel_group_update_selection_permissions (self);
-_ti_page_channel_group_update_button_remove_sensitivity (self);
+  _ti_page_channel_group_update_selection_permissions (self);
+  _ti_page_channel_group_update_button_remove_sensitivity (self);
 }
 
+
+static void
+_ti_page_channel_group_self_handle_changed (TIPageChannelGroup *self)
+{
+  TIPageChannelGroupPrivate *priv = TI_PAGE_CHANNEL_GROUP_GET_PRIVATE (self);
+  gchar *str;
+  GtkTextIter iter;
+
+  str = g_strdup_printf ("SelfHandle = %u",
+     tp_channel_group_get_self_handle (priv->channel));
+  gtk_text_buffer_get_end_iter (priv->textbuffer_event_log, &iter);
+  gtk_text_buffer_insert (priv->textbuffer_event_log, &iter, str, -1);
+  g_free (str);
+}
+
+
 TIPageChannelGroup *
 ti_page_channel_group_new (GtkNotebook *parent_notebook,
-                         GtkWindow *parent_window,
-                         TpChannel *channel,
-                         TIHandleMapper *handle_mapper)
+                           GtkWindow *parent_window,
+                           TpChannel *channel,
+                           TIHandleMapper *handle_mapper)
 {
-TIPage *self = NULL;
-TIPageChannelGroupPrivate *priv = NULL;
+  TIPage *self = NULL;
+  TIPageChannelGroupPrivate *priv = NULL;
 
-g_assert (channel != NULL);
+  g_assert (channel != NULL);
+  g_return_val_if_fail (tp_channel_is_ready (channel), NULL);
 
-self = g_object_new (TI_TYPE_PAGE_CHANNEL_GROUP, NULL);
-priv = TI_PAGE_CHANNEL_GROUP_GET_PRIVATE (self);
+  self = g_object_new (TI_TYPE_PAGE_CHANNEL_GROUP, NULL);
+  priv = TI_PAGE_CHANNEL_GROUP_GET_PRIVATE (self);
 
-priv->channel = g_object_ref (channel);
-priv->handle_mapper = g_object_ref (handle_mapper);
+  priv->channel = g_object_ref (channel);
+  priv->handle_mapper = g_object_ref (handle_mapper);
 
-tp_cli_channel_interface_group_connect_to_members_changed (priv->channel,
-    _ti_page_channel_group_members_changed, NULL, NULL,
-    (GObject *) self, NULL);
-tp_cli_channel_interface_group_connect_to_group_flags_changed (priv->channel,
-    _ti_page_channel_group_group_flags_changed, NULL, NULL,
-    (GObject *) self, NULL);
+  tp_cli_channel_interface_group_connect_to_members_changed (priv->channel,
+      _ti_page_channel_group_members_changed, NULL, NULL,
+      (GObject *) self, NULL);
 
-g_signal_connect_swapped (priv->preferences, "handle-display-mode-changed",
-    G_CALLBACK (_ti_page_channel_group_handle_display_mode_changed),
-    self);
+  tp_cli_channel_interface_group_connect_to_group_flags_changed (priv->channel,
+      _ti_page_channel_group_group_flags_changed, NULL, NULL,
+      (GObject *) self, NULL);
 
-priv->dialog_add_members = ti_dlg_add_members_new (parent_window,
-    handle_mapper);
+  g_signal_connect_swapped (priv->preferences, "handle-display-mode-changed",
+      G_CALLBACK (_ti_page_channel_group_handle_display_mode_changed),
+      self);
 
-if (priv->dialog_add_members == NULL)
-  {
-    g_object_unref (self);
-    self = NULL;
-    goto CLEAN_UP;
-  }
+  priv->dialog_add_members = ti_dlg_add_members_new (parent_window,
+      handle_mapper);
 
-priv->dialog_get_text = ti_dlg_get_text_new (parent_window,
-    "Enter message:");
+  if (priv->dialog_add_members == NULL)
+    {
+      g_object_unref (self);
+      self = NULL;
+      goto CLEAN_UP;
+    }
 
-if (priv->dialog_get_text == NULL)
-  {
-    g_object_unref (self);
-    self = NULL;
-    goto CLEAN_UP;
-  }
+  priv->dialog_get_text = ti_dlg_get_text_new (parent_window,
+      "Enter message:");
 
-_ti_page_new (&self, parent_notebook,
-    "page-channel-group.xml");
+  if (priv->dialog_get_text == NULL)
+    {
+      g_object_unref (self);
+      self = NULL;
+      goto CLEAN_UP;
+    }
+
+  _ti_page_new (&self, parent_notebook,
+      "page-channel-group.xml");
 
 CLEAN_UP:
-  return (TIPageChannelGroup *) self;
+    return (TIPageChannelGroup *) self;
 }
 
 /**
@@ -294,17 +315,17 @@ CLEAN_UP:
 static void
 _ti_page_channel_group_setup_page (TIPage *page, GladeXML *glade_xml)
 {
-TIPageChannelGroup *self = TI_PAGE_CHANNEL_GROUP (page);
-TIPageChannelGroupPrivate *priv = TI_PAGE_CHANNEL_GROUP_GET_PRIVATE (self);
-gboolean can_add;
-GtkWidget *widget;
+  TIPageChannelGroup *self = TI_PAGE_CHANNEL_GROUP (page);
+  TIPageChannelGroupPrivate *priv = TI_PAGE_CHANNEL_GROUP_GET_PRIVATE (self);
+  gboolean can_add;
+  GtkWidget *widget;
 
-_ti_page_channel_group_setup_group_flags (self, glade_xml);
+  _ti_page_channel_group_setup_group_flags (self, glade_xml);
 
-_ti_page_channel_group_build_members_treeview (self, glade_xml);
-_ti_page_channel_group_fill_members_list (self);
+  _ti_page_channel_group_build_members_treeview (self, glade_xml);
+  _ti_page_channel_group_fill_members_list (self);
 
-priv->button_remove_members = GTK_BUTTON (glade_xml_get_widget (glade_xml,
+  priv->button_remove_members = GTK_BUTTON (glade_xml_get_widget (glade_xml,
         "button_remove_members"));
   g_assert (GTK_IS_BUTTON (priv->button_remove_members));
   g_signal_connect_swapped (priv->button_remove_members, "clicked",
@@ -333,6 +354,10 @@ priv->button_remove_members = GTK_BUTTON (glade_xml_get_widget (glade_xml,
   g_assert (GTK_IS_BUTTON (widget));
   g_signal_connect_swapped (GTK_BUTTON (widget), "clicked",
       G_CALLBACK (_ti_page_channel_group_clear_log), self);
+
+  g_signal_connect_swapped (priv->channel, "notify::group-self-handle",
+      G_CALLBACK (_ti_page_channel_group_self_handle_changed), self);
+  _ti_page_channel_group_self_handle_changed ((TIPageChannelGroup *) self);
 }
 
 static void
@@ -348,8 +373,12 @@ _ti_page_channel_group_build_members_treeview (TIPageChannelGroup *self,
   g_assert (priv->members_tree_view != NULL &&
       GTK_IS_TREE_VIEW (priv->members_tree_view));
 
-  priv->members_list = gtk_list_store_new (3, G_TYPE_UINT /*handle*/,
-      G_TYPE_STRING /*name*/, G_TYPE_STRING /*status*/);
+  priv->members_list = gtk_list_store_new (5,
+      G_TYPE_UINT /*handle*/,
+      G_TYPE_STRING /*name*/,
+      G_TYPE_STRING /*status*/,
+      G_TYPE_UINT /*handle owner handle*/,
+      G_TYPE_STRING /*handle owner name*/);
   gtk_tree_view_set_model (priv->members_tree_view,
       GTK_TREE_MODEL (priv->members_list));
 
@@ -378,6 +407,23 @@ _ti_page_channel_group_build_members_treeview (TIPageChannelGroup *self,
       NULL);
   gtk_tree_view_append_column (priv->members_tree_view, column);
 
+  if (ti_preferences_get_handle_display_mode (priv->preferences) ==
+      TI_PREFERENCES_HANDLE_DISPLAY_HANDLE)
+    {
+      column = gtk_tree_view_column_new_with_attributes ("Owner handle",
+          renderer,
+          "text", 3,
+          NULL);
+      gtk_tree_view_append_column (priv->members_tree_view, column);
+    }
+  else
+    {
+      column = gtk_tree_view_column_new_with_attributes ("Owner", renderer,
+          "text", 4,
+          NULL);
+      gtk_tree_view_append_column (priv->members_tree_view, column);
+    }
+
   priv->members_selection = gtk_tree_view_get_selection (
       priv->members_tree_view);
 
@@ -395,17 +441,12 @@ _ti_page_channel_group_fill_members_list (TIPageChannelGroup *self)
   TIPageChannelGroupPrivate *priv =
       TI_PAGE_CHANNEL_GROUP_GET_PRIVATE (self);
   GArray *members, *lp, *rp;
-  GError *error = NULL;
 
   gtk_list_store_clear (priv->members_list);
 
-  if (!tp_cli_channel_interface_group_run_get_all_members (priv->channel,
-        -1, &members, &lp, &rp, &error, NULL))
-    {
-      g_printerr ("GetAllMembers(): %s\n", error->message);
-      g_error_free (error);
-      return;
-    }
+  members = tp_intset_to_array (tp_channel_group_get_members (priv->channel));
+  lp = tp_intset_to_array (tp_channel_group_get_local_pending (priv->channel));
+  rp = tp_intset_to_array (tp_channel_group_get_remote_pending (priv->channel));
 
   _ti_page_channel_group_append_members_to_list (self, members,
       TI_GROUP_MEMBER_STATUS_REGULAR);
@@ -425,22 +466,43 @@ _ti_page_channel_group_append_members_to_list (TIPageChannelGroup *self,
                                                const gchar *status)
 {
   TIPageChannelGroupPrivate *priv = TI_PAGE_CHANNEL_GROUP_GET_PRIVATE (self);
-  GtkTreeIter iter;
   guint i;
-  guint handle;
-  gchar *name;
 
   for (i = 0; i < handles->len; i++)
     {
+      GtkTreeIter iter;
+      guint handle, owner;
+      gchar *name, *oname;
+      gchar *specific_status;
+      TpHandle actor;
+      TpChannelGroupChangeReason reason;
+      const gchar *message;
+
       handle = g_array_index (handles, guint, i);
+      owner = tp_channel_group_get_handle_owner (priv->channel, handle);
 
       name = ti_handle_mapper_get_contact_handle_name (priv->handle_mapper,
           handle);
+      oname = ti_handle_mapper_get_contact_handle_name (priv->handle_mapper,
+          owner);
 
       if (name == NULL)
+        name = g_strdup_printf ("%u", handle);
+
+      if (oname == NULL)
+        oname = g_strdup_printf ("%u", owner);
+
+      if (tp_channel_group_get_local_pending_info (priv->channel, handle,
+            &actor, &reason, &message))
         {
-          name = g_strdup_printf ("%u", handle);
+          specific_status = g_strdup_printf ("%s: actor %u, reason %u, \"%s\"",
+              status, actor, reason, message);
         }
+      else
+        {
+          specific_status = g_strdup (status);
+        }
+
 
       /* If there's not already a row for that contact, we create one. */
       if (!_ti_page_channel_get_contact_row (self, handle, &iter))
@@ -451,10 +513,14 @@ _ti_page_channel_group_append_members_to_list (TIPageChannelGroup *self,
       gtk_list_store_set (priv->members_list, &iter,
           TI_COLUMN_MEMBERS_LIST_HANDLE, handle,
           TI_COLUMN_MEMBERS_LIST_NAME, name,
-          TI_COLUMN_MEMBERS_LIST_STATUS, status,
+          TI_COLUMN_MEMBERS_LIST_STATUS, specific_status,
+          TI_COLUMN_MEMBERS_LIST_OWNER, owner,
+          TI_COLUMN_MEMBERS_LIST_OWNER_NAME, oname,
           -1);
 
       g_free (name);
+      g_free (oname);
+      g_free (specific_status);
     }
 }
 
@@ -463,7 +529,6 @@ _ti_page_channel_group_setup_group_flags (TIPageChannelGroup *self,
                                           GladeXML *glade_xml)
 {
   TIPageChannelGroupPrivate *priv = TI_PAGE_CHANNEL_GROUP_GET_PRIVATE (self);
-  GError *error = NULL;
 
   priv->checkbutton_can_add = glade_xml_get_widget (glade_xml,
       "checkbutton_can_add");
@@ -501,14 +566,7 @@ _ti_page_channel_group_setup_group_flags (TIPageChannelGroup *self,
       "checkbutton_channel_specific_handles");
   g_assert (GTK_IS_CHECK_BUTTON (priv->checkbutton_channel_specific_handles));
 
-  if (!tp_cli_channel_interface_group_run_get_group_flags (priv->channel,
-        -1, &priv->group_flags, &error, NULL))
-    {
-      g_printerr ("GetGroupFlags(): %s\n", error->message);
-      g_error_free (error);
-      error = NULL;
-      priv->group_flags = 0;
-    }
+  priv->group_flags = tp_channel_group_get_flags (priv->channel);
 
   _ti_page_channel_group_refresh_group_flags_checkboxes (self);
 }
@@ -919,6 +977,26 @@ _ti_page_channel_group_handle_display_mode_changed (TIPageChannelGroup *self,
           NULL);
     }
 
+  contact_handle_column = gtk_tree_view_get_column (priv->members_tree_view,
+      2);
+
+  if (handle_display_mode == TI_PREFERENCES_HANDLE_DISPLAY_HANDLE)
+    {
+      gtk_tree_view_column_set_title (contact_handle_column, "Owner handle");
+      gtk_tree_view_column_set_attributes (contact_handle_column, renderer,
+          "text", TI_COLUMN_MEMBERS_LIST_OWNER,
+          NULL);
+    }
+  else
+    {
+      g_assert (handle_display_mode == TI_PREFERENCES_HANDLE_DISPLAY_NAME);
+
+      gtk_tree_view_column_set_title (contact_handle_column, "Owner");
+      gtk_tree_view_column_set_attributes (contact_handle_column, renderer,
+          "text", TI_COLUMN_MEMBERS_LIST_OWNER_NAME,
+          NULL);
+    }
+
   g_list_free (renderers_list);
 }
 
