@@ -42,7 +42,9 @@
 #include "dlg-request-channel.h"
 #include "handle-mapper.h"
 #include "page-aliasing.h"
+#include "page-contacts.h"
 #include "page-capabilities.h"
+#include "page-simple-presence.h"
 #include "page-presence.h"
 #include "page-properties.h"
 #include "util.h"
@@ -125,7 +127,9 @@ struct _TIWndConnectionPrivate {
     GPtrArray *channel_wnds;
 
     TIPageAliasing *page_aliasing;
+    TIPageContacts *page_contacts;
     TIPageCapabilities *page_capabilities;
+    TIPageSimplePresence *page_simple_presence;
     TIPagePresence *page_presence;
     TIPageProperties *page_properties;
 };
@@ -174,7 +178,9 @@ ti_wnd_connection_dispose (GObject *object)
     }
 
   TI_OBJ_UNREF (self->priv->page_aliasing);
+  TI_OBJ_UNREF (self->priv->page_contacts);
   TI_OBJ_UNREF (self->priv->page_capabilities);
+  TI_OBJ_UNREF (self->priv->page_simple_presence);
   TI_OBJ_UNREF (self->priv->page_presence);
   TI_OBJ_UNREF (self->priv->page_properties);
 
@@ -370,7 +376,9 @@ ti_wnd_connection_init (TIWndConnection *self)
   self->priv->notebook = NULL;
   self->priv->channel_wnds = g_ptr_array_new ();
   self->priv->page_aliasing = NULL;
+  self->priv->page_contacts = NULL;
   self->priv->page_capabilities = NULL;
+  self->priv->page_simple_presence = NULL;
   self->priv->page_presence = NULL;
   self->priv->page_properties = NULL;
 }
@@ -1316,6 +1324,17 @@ _ti_wnd_connection_create_interface_page (TIWndConnection *self, gchar *interfac
           ti_page_attach_to_notebook (TI_PAGE (self->priv->page_presence), self->priv->notebook);
         }
     }
+  else if (g_str_equal (interface_name, "org.freedesktop.Telepathy.Connection.Interface.SimplePresence"))
+    {
+      if (self->priv->page_simple_presence == NULL)
+        {
+          self->priv->page_simple_presence = ti_page_simple_presence_new (GTK_WINDOW (self->priv->window), self->priv->notebook, self->priv->connection, self->priv->handle_mapper);
+        }
+        else
+        {
+          ti_page_attach_to_notebook (TI_PAGE (self->priv->page_simple_presence), self->priv->notebook);
+        }
+    }
   else if (g_str_equal (interface_name, "org.freedesktop.Telepathy.Connection.Interface.Aliasing"))
     {
       if (self->priv->page_aliasing == NULL)
@@ -1327,6 +1346,17 @@ _ti_wnd_connection_create_interface_page (TIWndConnection *self, gchar *interfac
           ti_page_attach_to_notebook (TI_PAGE (self->priv->page_aliasing), self->priv->notebook);
         }
     }
+  else if (g_str_equal (interface_name, "org.freedesktop.Telepathy.Connection.Interface.Contacts"))
+    {
+      if (self->priv->page_contacts == NULL)
+        {
+          self->priv->page_contacts = ti_page_contacts_new (GTK_WINDOW (self->priv->window), self->priv->notebook, self->priv->connection, self->priv->handle_mapper);
+        }
+      else
+        {
+          ti_page_attach_to_notebook (TI_PAGE (self->priv->page_contacts), self->priv->notebook);
+        }
+    }
   else if (g_str_equal (interface_name, "org.freedesktop.Telepathy.Connection.Interface.Capabilities"))
     {
       if (self->priv->page_capabilities == NULL)
